﻿using System.Configuration;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace Salient6.MvcSamples.Common.Configuration.Tests {

    [TestClass]
    public class AliasRoutesElementCollectionTest {

        [TestMethod]
        public void IndexerReturnsElement() {
            AliasRoutesSection section = ConfigurationManager.GetSection(AliasRoutesSection.SectionName) as AliasRoutesSection;
            AliasRoutesElementCollection target = section.Routes;

            Assert.IsNotNull(target[0]);
        }

        [TestMethod]
        public void CreateNewElementReturnsNew() {
            AliasRoutesElementCollection_Accessor target = new AliasRoutesElementCollection_Accessor();

            Assert.IsInstanceOfType(target.CreateNewElement(), typeof(AliasRoutesElement));
        }

        [TestMethod]
        public void CreateElementKeyReturnsConcatOfTokenAndValue() {
            AliasRoutesSection section = ConfigurationManager.GetSection(AliasRoutesSection.SectionName) as AliasRoutesSection;
            AliasRoutesElementCollection routes = section.Routes;
            AliasRoutesElement element = routes[0];
            AliasRoutesElementCollection_Accessor target = AliasRoutesElementCollection_Accessor.AttachShadow(routes);

            string expected = "controller-home";
            string actual = target.GetElementKey(element).ToString();

            Assert.AreEqual<string>(expected, actual);
        }

        [TestMethod]
        public void GetElementByAliasReturnsFoundElement() {
            AliasRoutesSection section = ConfigurationManager.GetSection(AliasRoutesSection.SectionName) as AliasRoutesSection;
            AliasRoutesElementCollection target = section.Routes;
            AliasRoutesElement actual = target.GetElementByAlias("controller", "start");

            Assert.IsNotNull(actual);
        }

        [TestMethod]
        public void GetElementByAliasReturnsNullWhenTokenIsNotFound() {
            AliasRoutesSection section = ConfigurationManager.GetSection(AliasRoutesSection.SectionName) as AliasRoutesSection;
            AliasRoutesElementCollection target = section.Routes;
            AliasRoutesElement actual = target.GetElementByAlias("nonexistent", "start");

            Assert.IsNull(actual);
        }

        [TestMethod]
        public void GetElementByAliasReturnsNullWhenAliasIsNotFound() {
            AliasRoutesSection section = ConfigurationManager.GetSection(AliasRoutesSection.SectionName) as AliasRoutesSection;
            AliasRoutesElementCollection target = section.Routes;
            AliasRoutesElement actual = target.GetElementByAlias("controller", "nonexistent");

            Assert.IsNull(actual);
        }

        [TestMethod]
        public void GetValueReturnsValue() {
            AliasRoutesSection section = ConfigurationManager.GetSection(AliasRoutesSection.SectionName) as AliasRoutesSection;
            AliasRoutesElementCollection target = section.Routes;

            string expected = "home";
            string actual = target.GetValue("controller", "start");

            Assert.AreEqual<string>(expected, actual);
        }

        [TestMethod]
        public void GetValueReturnsNullWhenElementNotFound() {
            AliasRoutesSection section = ConfigurationManager.GetSection(AliasRoutesSection.SectionName) as AliasRoutesSection;
            AliasRoutesElementCollection target = section.Routes;

            Assert.IsNull(target.GetValue("nonexistent", "start"));
        }
    }
}
