﻿using System.Collections;
using System.Collections.Generic;
using System.Configuration;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace Salient6.MvcSamples.Common.Configuration.Tests {

    [TestClass]
    public class AliasRoutesElementTest {

        [TestMethod]
        public void Token() {
            AliasRoutesSection section = ConfigurationManager.GetSection(AliasRoutesSection.SectionName) as AliasRoutesSection;
            AliasRoutesElement target = section.Routes[0];

            string expected = "controller";
            string actual = target.Token;

            Assert.AreEqual<string>(expected, actual);
        }

        [TestMethod]
        public void Value() {
            AliasRoutesSection section = ConfigurationManager.GetSection(AliasRoutesSection.SectionName) as AliasRoutesSection;
            AliasRoutesElement target = section.Routes[0];

            string expected = "home";
            string actual = target.Value;

            Assert.AreEqual<string>(expected, actual);
        }

        [TestMethod]
        public void AliasesRawValue() {
            AliasRoutesSection section = ConfigurationManager.GetSection(AliasRoutesSection.SectionName) as AliasRoutesSection;
            AliasRoutesElement target = section.Routes[0];

            string expected = "inicio, beginning, baile, start";
            string actual = target.AliasesRawValue;

            Assert.AreEqual<string>(expected, actual);
        }

        [TestMethod]
        public void Aliases() {
            AliasRoutesSection section = ConfigurationManager.GetSection(AliasRoutesSection.SectionName) as AliasRoutesSection;
            AliasRoutesElement target = section.Routes[0];

            ICollection expected = new List<string>(new string[] { "inicio", "beginning", "baile", "start" });
            ICollection actual = target.Aliases as ICollection;

            CollectionAssert.AreEquivalent(expected, actual);
        }

        [TestMethod]
        public void AliasesIsEmpty() {
            AliasRoutesSection section = ConfigurationManager.GetSection(AliasRoutesSection.SectionName) as AliasRoutesSection;
            AliasRoutesElement target = section.Routes[1];

            ICollection expected = new List<string>();
            ICollection actual = target.Aliases as ICollection;

            CollectionAssert.AreEquivalent(expected, actual);
        }

        [TestMethod]
        public void ContainsAliasReturnsTrueRegardlessOfCase() {
            AliasRoutesSection section = ConfigurationManager.GetSection(AliasRoutesSection.SectionName) as AliasRoutesSection;
            AliasRoutesElement target = section.Routes[0];

            Assert.IsTrue(target.ContainsAlias("StArT"));
        }
    }
}
