﻿using System.Configuration;
using Salient6.MvcSamples.Common;
using Salient6.MvcSamples.Common.Extensions;

namespace Salient6.MvcSamples.Common.Configuration {

    public class AliasRoutesElementCollection : ConfigurationElementCollection {

        public AliasRoutesElement this[int index] {
            get {
                return base.BaseGet(index) as AliasRoutesElement;
            }
        }

        public AliasRoutesElement GetElementByAlias(string token, string alias) {
            Guard.AgainstNullParameter(token, "token");
            Guard.AgainstNullParameter(alias, "alias");

            for(int i = 0; i < this.Count; i++) {
                AliasRoutesElement element = this[i];
                bool exists = token.IsEqualTo(element.Token) && element.ContainsAlias(alias);

                if(exists) return element;
            }

            return null; 
        }

        public string GetValue(string token, string alias) {
            AliasRoutesElement element = this.GetElementByAlias(token, alias);

            return element != null ? element.Value : null;
        }

        protected override ConfigurationElement CreateNewElement() {
            return new AliasRoutesElement();
        }

        protected override object GetElementKey(ConfigurationElement element) {
            Guard.AgainstNullParameter(element, "element");

            AliasRoutesElement item = element as AliasRoutesElement;
            string key = string.Concat(item.Token, "-", item.Value).ToLower();

            return key;
        }
    }
}

