﻿using System;
using System.Web;
using System.Web.Routing;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Moq;

namespace Salient6.MvcSamples.Web.Modules.Tests {

    [TestClass]
    public class AliasRoutingModuleTest {

        [TestMethod]
        public void GetRouteDataDefaultIsNotNull() {
            AliasRoutingModule target = new AliasRoutingModule();
            Func<HttpContextBase, RouteData> func = target.GetRouteData;

            Assert.IsNotNull(target);
        }

        [TestMethod]
        public void PostMapRequestHandlerReturnsWhenItemIsNull() {
            AliasRoutingModule target = new AliasRoutingModule();
            Mock<HttpContextBase> mockContext = new Mock<HttpContextBase>();

            mockContext.SetupGet(c => c.Items[It.IsAny<object>()]).Returns(null).Verifiable();

            target.PostMapRequestHandler(mockContext.Object);

            mockContext.VerifyAll();
        }

        [TestMethod]
        public void PostMapRequestHandlerAssignsContextProperties() {
            AliasRoutingModule target = new AliasRoutingModule();
            Mock<HttpContextBase> mockContext = new Mock<HttpContextBase>();
            AliasRoutingModule.RequestInfo info = new AliasRoutingModule.RequestInfo {
                HttpHandler = new Mock<IHttpHandler>().Object,
                OriginalPath = "path"
            };

            mockContext.SetupGet(c => c.Items[It.IsAny<object>()]).Returns(info).Verifiable();
            mockContext.SetupSet(c => c.Handler = It.IsAny<IHttpHandler>()).Verifiable();
            mockContext.Setup(c => c.RewritePath(It.IsAny<string>())).Verifiable();

            target.PostMapRequestHandler(mockContext.Object);

            mockContext.VerifyAll();
        }

        [TestMethod]
        public void PostResolveRequestCacheReturnsWhenRouteDataIsNull() {
            AliasRoutingModule target = new AliasRoutingModule();
            Mock<HttpContextBase> mockContext = new Mock<HttpContextBase>();

            target.GetRouteData = c => null;
            target.PostResolveRequestCache(mockContext.Object);
        }

        [TestMethod, ExpectedException(typeof(InvalidOperationException))]
        public void PostResolveRequestCacheThrowsExceptionWhenHandlerIsNull() {
            AliasRoutingModule target = new AliasRoutingModule();
            Mock<HttpContextBase> mockContext = new Mock<HttpContextBase>();

            target.GetRouteData = c => new RouteData();
            target.PostResolveRequestCache(mockContext.Object);
        }

        [TestMethod]
        public void PostResolveRequestCacheReturnsWhenHandlerIsStopHandler() {
            AliasRoutingModule target = new AliasRoutingModule();
            Mock<HttpContextBase> mockContext = new Mock<HttpContextBase>();
            RouteData data = new RouteData();
            data.RouteHandler = new StopRoutingHandler();

            target.GetRouteData = c => data;
            target.PostResolveRequestCache(mockContext.Object);
        }

        [TestMethod, ExpectedException(typeof(InvalidOperationException))]
        public void PostResolveRequestCacheAThrowsExceptionWhenHttpHandlerIsNull() {
            AliasRoutingModule target = new AliasRoutingModule();
            Mock<HttpContextBase> mockContext = new Mock<HttpContextBase>();
            Mock<HttpRequestBase> mockRequest = new Mock<HttpRequestBase>();
            Mock<IRouteHandler> mockRouteHandler = new Mock<IRouteHandler>();
            RouteData data = new RouteData();

            mockContext.SetupGet(c => c.Request).Returns(mockRequest.Object);
            mockContext.SetupGet(c => c.Items[It.IsAny<object>()]).Returns((AliasRoutingModule.RequestInfo)null);
            mockRequest.SetupGet(r => r.Path).Returns(string.Empty);
            mockRouteHandler.Setup(rh => rh.GetHttpHandler(It.IsAny<RequestContext>())).Returns((IHttpHandler)null);
            data.RouteHandler = mockRouteHandler.Object;

            target.GetRouteData = c => data;
            target.PostResolveRequestCache(mockContext.Object);
        }

        [TestMethod]
        public void PostResolveRequestCacheAssignsAliasValues() {
            AliasRoutingModule target = new AliasRoutingModule();
            Mock<HttpContextBase> mockContext = new Mock<HttpContextBase>();
            Mock<HttpRequestBase> mockRequest = new Mock<HttpRequestBase>();
            Mock<IRouteHandler> mockRouteHandler = new Mock<IRouteHandler>();
            RouteData data = new RouteData();

            mockContext.SetupGet(c => c.Request).Returns(mockRequest.Object);
            mockContext.SetupGet(c => c.Items[It.IsAny<object>()]).Returns((AliasRoutingModule.RequestInfo)null);
            mockRequest.SetupGet(r => r.Path).Returns(string.Empty);
            mockRouteHandler.Setup(rh => rh.GetHttpHandler(It.IsAny<RequestContext>())).Returns(new Mock<IHttpHandler>().Object);
            data.RouteHandler = mockRouteHandler.Object;
            data.Values.Add("controller", "start");

            target.GetRouteData = c => data;
            target.PostResolveRequestCache(mockContext.Object);

            string expected = "home";
            string actual = data.Values["controller"].ToString();

            Assert.AreEqual<string>(expected, actual);
        }
    }
}
