﻿using System;
using System.Collections.Generic;
using System.Web;
using System.Web.Routing;
using Salient6.MvcSamples.Common;
using Salient6.MvcSamples.Common.Configuration;

namespace Salient6.MvcSamples.Web.Modules {

    public class AliasRoutingModule : UrlRoutingModule {
        protected static readonly object itemsKey = new object();

        public class RequestInfo {
            public string OriginalPath { get; set; }
            public IHttpHandler HttpHandler { get; set; }
        }

        public Func<HttpContextBase, RouteData> GetRouteData {
            get {
                if(this._getRouteData == null) {
                    this._getRouteData = this.RouteCollection.GetRouteData;
                }

                return this._getRouteData;
            }
            set {
                this._getRouteData = value;
            }
        }   private Func<HttpContextBase, RouteData> _getRouteData;

        public override void PostMapRequestHandler(HttpContextBase context) {
            Guard.AgainstNullParameter(context, "context");

            RequestInfo info = context.Items[itemsKey] as RequestInfo;
            if(info == null) return;

            context.RewritePath(info.OriginalPath);
            context.Handler = info.HttpHandler;
        }

        public override void PostResolveRequestCache(HttpContextBase context) {
            Guard.AgainstNullParameter(context, "context");

            RouteData routeData = this.GetRouteData(context);
            if(routeData == null) return;

            IRouteHandler routeHandler = routeData.RouteHandler;
            if(routeHandler == null) throw new InvalidOperationException("No RouteHandler identified");
            if(routeHandler is StopRoutingHandler) return;

            IDictionary<string, object> values = new Dictionary<string, object>(routeData.Values);
            AliasRoutesSection section = SectionRepository.Current.AliasRoutes;

            //*** this is where the magic happens
            foreach(string token in values.Keys) {
                string alias = values[token].ToString();
                string value = section.Routes.GetValue(token, alias);

                if(value != null) routeData.Values[token] = value;
            }

            RequestContext requestContext = new RequestContext(context, routeData);
            IHttpHandler httpHandler = routeHandler.GetHttpHandler(requestContext);
            if(httpHandler == null) throw new InvalidOperationException("No HttpHandler identified");

            RequestInfo info = new RequestInfo { OriginalPath = context.Request.Path, HttpHandler = httpHandler };

            context.Items[itemsKey] = info;
            context.RewritePath("~/UrlRouting.axd");
        }
    }
}
